package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.PageRequest;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/1 11:37 上午
 */
@Slf4j
@Service
public class ChuanyunProjectScheduledTask {

    private final Integer pageSize=20; final ObjectMapper objectMapper=new ObjectMapper();
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunSaleContractDao chuanyunSaleContractDao;
    private final ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao;
    private final ChuanyunPurCostDao chuanyunPurCostDao;
    private final ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao;
    private final ChuanyunReimbursementDao chuanyunReimbursementDao;
    private final ChuanyunPersonDaysDao chuanyunPersonDaysDao;
    private final ChuanyunProjectDao chuanyunProjectDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunWorkHourDao chuanyunWorkHourDao;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final ChuanyunPayrollDao chuanyunPayrollDao;
    private final ChuanyunSelfCostDao chuanyunSelfCostDao;
    private final ChuanyunMemberCostDao chuanyunMemberCostDao;
    private final ChuanyunProjectCostDao chuanyunProjectCostDao;
    private final ChuanyunReimbursementAmountDao chuanyunReimbursementAmountDao;
    private final ChuanyunMonthLaborCostDao chuanyunMonthLaborCostDao;

    public ChuanyunProjectScheduledTask(ChuanYunManager chuanYunManager, ChuanyunSaleContractDao chuanyunSaleContractDao, ChuanyunPaymentReceiveDao chuanyunPaymentReceiveDao, ChuanyunPurCostDao chuanyunPurCostDao, ChuanyunProjectCashFlowDao chuanyunProjectCashFlowDao, ChuanyunReimbursementDao chuanyunReimbursementDao, ChuanyunPersonDaysDao chuanyunPersonDaysDao, ChuanyunProjectDao chuanyunProjectDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunWorkHourDao chuanyunWorkHourDao, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, ChuanyunPayrollDao chuanyunPayrollDao, ChuanyunSelfCostDao chuanyunSelfCostDao, ChuanyunMemberCostDao chuanyunMemberCostDao, ChuanyunProjectCostDao chuanyunProjectCostDao, ChuanyunReimbursementAmountDao chuanyunReimbursementAmountDao, ChuanyunMonthLaborCostDao chuanyunMonthLaborCostDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunSaleContractDao = chuanyunSaleContractDao;
        this.chuanyunPaymentReceiveDao = chuanyunPaymentReceiveDao;
        this.chuanyunPurCostDao = chuanyunPurCostDao;
        this.chuanyunProjectCashFlowDao = chuanyunProjectCashFlowDao;
        this.chuanyunReimbursementDao = chuanyunReimbursementDao;
        this.chuanyunPersonDaysDao = chuanyunPersonDaysDao;
        this.chuanyunProjectDao = chuanyunProjectDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunWorkHourDao = chuanyunWorkHourDao;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.chuanyunPayrollDao = chuanyunPayrollDao;
        this.chuanyunSelfCostDao = chuanyunSelfCostDao;
        this.chuanyunMemberCostDao = chuanyunMemberCostDao;
        this.chuanyunProjectCostDao = chuanyunProjectCostDao;
        this.chuanyunReimbursementAmountDao = chuanyunReimbursementAmountDao;
        this.chuanyunMonthLaborCostDao = chuanyunMonthLaborCostDao;
    }

    /**
     * 更新销售合同数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSaleContractDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSaleContractDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同信息保存保存"+totalCount);
    }

    /**
     * 同步项目收付款
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getLaborCost() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMonthLaborCostDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunMonthLaborCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunMonthLaborCostDO -> {
                    var tmp=chuanyunMonthLaborCostDao.findById(chuanyunMonthLaborCostDO.getObjectId());
                    tmp.ifPresent(chuanyunMonthLaborCostDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunMonthLaborCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目人力成本保存"+totalCount);
    }

    /**
     * 同步项目收付款
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getCashFlow() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectCashFlowDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunProjectCashFlowDO -> {
                    var tmp=chuanyunProjectCashFlowDao.findById(chuanyunProjectCashFlowDO.getObjectId());
                    tmp.ifPresent(chuanyunProjectCashFlowDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectCashFlowDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目收付款保存"+totalCount);
    }

    /**
     * 同步项目物料成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getPurCost() {
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunPurCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPurCostDO -> {
                    var tmp=chuanyunPurCostDao.findById(chuanyunPurCostDO.getObjectId());
                    tmp.ifPresent(chuanyunPurCostDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPurCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
            }
        log.info("项目物料成本保存"+totalCount);
    }

    /**
     * 更新销售合同回款数据
     */
    @Scheduled(fixedDelay = 3600000L)
    private void getSaleContractPaymentReceiveFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPaymentReceiveDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunPaymentReceiveDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                result.forEach(chuanyunPaymentReceiveDO -> {
                    var tmp=chuanyunPaymentReceiveDao.findById(chuanyunPaymentReceiveDO.getObjectId());
                    tmp.ifPresent(chuanyunPaymentReceiveDao::delete);
                });
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunPaymentReceiveDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目销售合同回款信息保存保存"+totalCount);
    }


    /**
     * 人力成本
     */
    @Scheduled(fixedDelay = 3600000L)
    private void laborCost() {
        var objectMapper=new ObjectMapper();
        var flag = true;
        var pageSize = 20;
        var page = 0;
        while(flag) {
            var pageable= PageRequest.of(page,pageSize);
            var chuanyunUserCompanyList = chuanyunUserCompanyDao.findAllByStatus(1,pageable);
            if (page<=chuanyunUserCompanyList.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            //逐个将员工每天工时进行统计汇总
            List<ChuanyunUserCompanyDO> userCompanyDOList = chuanyunUserCompanyList.getContent();
            userCompanyDOList.forEach(chuanyunUserCompanyDO -> {
                //工时标准化
                standardHour(chuanyunUserCompanyDO);
                //单项目单人单月成本
                standardMonthlyHour(chuanyunUserCompanyDO);
            });
            //按项目汇总用户工时
            List<ChuanyunMemberCostDO> list = chuanyunMemberCostDao.getMemberCost();
            list.forEach(chuanyunMemberCostDO -> {
                if(StringUtils.isEmpty(chuanyunMemberCostDO.getProjectId())||StringUtils.isEmpty(chuanyunMemberCostDO.getUserId())){
                    log.info("########"+chuanyunMemberCostDO+"");
                }else{

                    BigDecimal standard = new BigDecimal(8);
                    ChuanyunMemberCostDO memberCostDO = new ChuanyunMemberCostDO();
                    memberCostDO.setCost(chuanyunMemberCostDO.getCost());
                    memberCostDO.setProjectType(chuanyunMemberCostDO.getProjectType());
                    memberCostDO.setProjectId(chuanyunMemberCostDO.getProjectId());
                    memberCostDO.setUserId(chuanyunMemberCostDO.getUserId());
                    memberCostDO.setTotalHour(chuanyunMemberCostDO.getTotalHour());
                    memberCostDO.setStatus(chuanyunMemberCostDO.getStatus());
                    memberCostDO.setPersonDays(chuanyunMemberCostDO.getTotalHour().divide(standard,2,RoundingMode.HALF_UP));
                    ChuanyunMemberCostDO oldCostDO = chuanyunMemberCostDao.findByUserIdAndProjectIdAndStatus(chuanyunMemberCostDO.getUserId(),chuanyunMemberCostDO.getProjectId(),chuanyunMemberCostDO.getStatus());
                    if (oldCostDO !=null) {
                        chuanyunMemberCostDao.delete(oldCostDO);
                    }
                    chuanyunMemberCostDao.save(memberCostDO);

                    //更新氚云数据
                    try {
                        ChuanyunProjectMemberCostDTO chuanyunProjectMemberCostDTO = new ChuanyunProjectMemberCostDTO();
                        chuanyunProjectMemberCostDTO.setProjectId(chuanyunMemberCostDO.getProjectId());
                        chuanyunProjectMemberCostDTO.setUserId(chuanyunMemberCostDO.getUserId());
                        chuanyunProjectMemberCostDTO.setCost(chuanyunMemberCostDO.getCost());
                        chuanyunProjectMemberCostDTO.setWorkHour(chuanyunMemberCostDO.getTotalHour());
                        chuanyunProjectMemberCostDTO.setProjectType(chuanyunMemberCostDO.getProjectType());
                        chuanyunProjectMemberCostDTO.setStatus(chuanyunMemberCostDO.getStatus());
                        chuanyunProjectMemberCostDTO.setPersonDays(chuanyunMemberCostDO.getTotalHour().divide(standard,2,RoundingMode.HALF_UP));
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000001_2,"+chuanyunMemberCostDO.getProjectId());
                        matchers.add("F0000007_2,"+chuanyunMemberCostDO.getStatus());
                        matchers.add("F0000002_2,"+chuanyunMemberCostDO.getUserId());
                        var filter= Filter.instance(0,1,true,"And",matchers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectMemberCostDO.SCHEMA_CODE,filter);
                        var projectMemberString =objectMapper.writeValueAsString(chuanyunProjectMemberCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        if (chuanyunFindAllResponse.getReturnData()!=null) {
                            List<ChuanyunProjectMemberCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunProjectMemberCostDO oldProjectMemberCostDO = result.get(0);
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunProjectMemberCostDO.SCHEMA_CODE,oldProjectMemberCostDO.getObjectId(), projectMemberString);
                        }else {
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunProjectMemberCostDO.SCHEMA_CODE, projectMemberString,true);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                        log.info("新增项目成员成本汇总成功");
                        }else{
                            log.warn("新增项目成员成本汇总失败");
                        }
                    } catch (JsonProcessingException | BigSizeException e) {
                        e.printStackTrace();
                        log.warn("########"+chuanyunMemberCostDO+"");
                    }
                }

            });
            //项目汇总
           List<ChuanyunProjectCostDO> list1 = chuanyunProjectCostDao.getProjectCost();
           list1.forEach(chuanyunProjectCostDO -> {
               //获取项目实施费用
               ChuanyunReimbursementAmountDO chuanyunReimbursementAmountDO = chuanyunReimbursementAmountDao.getAmount(chuanyunProjectCostDO.getProjectId());
               //更新数据库
               ChuanyunProjectCostDO projectCostDO = new ChuanyunProjectCostDO();
               projectCostDO.setCost(chuanyunProjectCostDO.getCost());
               projectCostDO.setProjectType(chuanyunProjectCostDO.getProjectType());
               projectCostDO.setProjectId(chuanyunProjectCostDO.getProjectId());
               projectCostDO.setTotalHour(chuanyunProjectCostDO.getTotalHour());
               projectCostDO.setStatus(chuanyunProjectCostDO.getStatus());
               projectCostDO.setPersonDays(chuanyunProjectCostDO.getPersonDays());
               if (chuanyunReimbursementAmountDO !=null) {
                   projectCostDO.setAmount(chuanyunReimbursementAmountDO.getTotalAmount());
               }
               ChuanyunProjectCostDO oldProjectDO = chuanyunProjectCostDao.findByProjectId(chuanyunProjectCostDO.getProjectId());
               if (oldProjectDO !=null) {
                   chuanyunProjectCostDao.delete(oldProjectDO);
               }
               chuanyunProjectCostDao.save(projectCostDO);
               try {
                   ChuanyunProjectTotalCostDTO chuanyunProjectTotalCostDTO = new ChuanyunProjectTotalCostDTO();
                   chuanyunProjectTotalCostDTO.setProjectType(chuanyunProjectCostDO.getProjectType());
                   chuanyunProjectTotalCostDTO.setProjectId(chuanyunProjectCostDO.getProjectId());
                   chuanyunProjectTotalCostDTO.setCost(chuanyunProjectCostDO.getCost());
                   chuanyunProjectTotalCostDTO.setStatus(chuanyunProjectCostDO.getStatus());
                   chuanyunProjectTotalCostDTO.setWorkHour(chuanyunProjectCostDO.getTotalHour());
                   chuanyunProjectTotalCostDTO.setPersonDays(chuanyunProjectCostDO.getPersonDays());
                   if (chuanyunReimbursementAmountDO !=null) {
                       chuanyunProjectTotalCostDTO.setAmount(chuanyunReimbursementAmountDO.getTotalAmount());
                   }
                   List<String> matchers = new ArrayList<>();
                   matchers.add("F0000001_2,"+chuanyunProjectCostDO.getProjectId());
                   matchers.add("F0000006_2,"+chuanyunProjectCostDO.getStatus());
                   var filter= Filter.instance(0,1,true,"And",matchers);
                   var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectTotalCostDO.SCHEMA_CODE,filter);
                   var projectTotalString=objectMapper.writeValueAsString(chuanyunProjectTotalCostDTO);
                   ChuanyunSaveDTO chuanyunSaveDTO;
                   if (chuanyunFindAllResponse.getReturnData()!=null) {
                       List<ChuanyunProjectTotalCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                       ChuanyunProjectTotalCostDO firstResult = result.get(0);
                       chuanyunSaveDTO = chuanYunManager.update(ChuanyunProjectTotalCostDO.SCHEMA_CODE,firstResult.getObjectId(),projectTotalString);
                   }else {
                       chuanyunSaveDTO = chuanYunManager.save(ChuanyunProjectTotalCostDO.SCHEMA_CODE,projectTotalString,true);

                   }
                   if(chuanyunSaveDTO.getSuccessful()){
                       log.info("新增项目成本汇总成功");
                   }else{
                       log.warn("新增项目成本汇总失败");
                   }
               } catch (JsonProcessingException | BigSizeException e) {
                   log.error(e.getMessage());
                   log.error(""+chuanyunProjectCostDO);
                   e.printStackTrace();
               }
           });
        }
    }

    /**
     * 员工单日、各个项目工时标准化,并保存
     * @param chuanyunUserCompanyDO 员工信息
     */
    private void standardHour(ChuanyunUserCompanyDO chuanyunUserCompanyDO) {
        BigDecimal standard = new BigDecimal(8);
        //每人每个项目每天工时
        List<ChuanyunWorkHourDO> selfList = chuanyunWorkHourDao.getSelfWorkHour(chuanyunUserCompanyDO.getUserId());
        selfList.forEach(self->{
            try{
                ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO = new ChuanyunSelfWorkHourDO();
                chuanyunSelfWorkHourDO.setStatus(self.getStatus());
                chuanyunSelfWorkHourDO.setUserId(chuanyunUserCompanyDO.getUserId());
                chuanyunSelfWorkHourDO.setProjectType(self.getProjectType());
                if(ChuanYunConstant.PRODUCTION_PROJECT.equals(self.getProjectType())){
                    chuanyunSelfWorkHourDO.setProjectId(ChuanYunConstant.PRODUCTION_PROJECT_ID);
                }else{
                    chuanyunSelfWorkHourDO.setProjectId(self.getProjectId());
                }
                chuanyunSelfWorkHourDO.setDayLogDate(self.getDayLogDate());
                //每人每天工时
                ChuanyunWorkHourDO chuanyunWorkHourDO = chuanyunWorkHourDao.getDayWorkHour(chuanyunUserCompanyDO.getUserId(),self.getDayLogDate());
                //工时标准化
                if (chuanyunWorkHourDO != null) {
                    if (chuanyunWorkHourDO.getHoursWorked().compareTo(standard)<0) {
                        chuanyunSelfWorkHourDO.setStandardWorkHour(self.getHoursWorked());
                    }else {
                        BigDecimal personDays = (self.getHoursWorked().divide(chuanyunWorkHourDO.getHoursWorked(), 2, RoundingMode.HALF_UP)).multiply(standard);
                        chuanyunSelfWorkHourDO.setStandardWorkHour(personDays);
                    }
                    ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO1 = chuanyunSelfWorkHourDao.findByUserIdAndAndProjectIdAndAndDayLogDate(chuanyunUserCompanyDO.getUserId(),self.getProjectId(),self.getDayLogDate());
                    if (chuanyunSelfWorkHourDO1!=null) {
                        chuanyunSelfWorkHourDao.delete(chuanyunSelfWorkHourDO1);
                    }
                    chuanyunSelfWorkHourDao.save(chuanyunSelfWorkHourDO);
//                log.info("工时标准化新增成功");
                    //更新氚云数据

                    ChuanyunSelfWorkHourDTO chuanyunSelfWorkHourDTO = new ChuanyunSelfWorkHourDTO();
                    if(ChuanYunConstant.PRODUCTION_PROJECT.equals(self.getProjectType())){
                        chuanyunSelfWorkHourDTO.setProjectId(ChuanYunConstant.PRODUCTION_PROJECT_ID);
                    }else{
                        chuanyunSelfWorkHourDTO.setProjectId(self.getProjectId());
                    }
                    chuanyunSelfWorkHourDTO.setUserId(chuanyunUserCompanyDO.getUserId());
                    chuanyunSelfWorkHourDTO.setDayLogDate(self.getDayLogDate());
                    chuanyunSelfWorkHourDTO.setProjectType(self.getProjectType());
                    chuanyunSelfWorkHourDTO.setStatus(self.getStatus());
                    chuanyunSelfWorkHourDTO.setDepartmentId(chuanyunUserCompanyDO.getDepartmentId());
                    chuanyunSelfWorkHourDTO.setStandardWorkHour(chuanyunSelfWorkHourDO.getStandardWorkHour());
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000001_2,"+self.getProjectId());
                    matchers.add("F0000002_2,"+chuanyunUserCompanyDO.getUserId());
                    matchers.add("F0000003_2,"+self.getDayLogDate());
                    matchers.add("F0000005_2,"+self.getProjectType());
                    matchers.add("F0000006_2,"+self.getStatus());
                    var filter= Filter.instance(0,1,true,"And",matchers);
                    var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunMemberHourDO.SCHEMA_CODE,filter);
                    var selfMonthString=objectMapper.writeValueAsString(chuanyunSelfWorkHourDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    if (chuanyunFindAllResponse.getReturnData()!=null){
                        List<ChuanyunMemberHourDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        ChuanyunMemberHourDO firstResult = result.get(0);
                        chuanyunSaveDTO =chuanYunManager.update(ChuanyunMemberHourDO.SCHEMA_CODE,firstResult.getObjectId(),selfMonthString);
                    }else {
                        chuanyunSaveDTO =chuanYunManager.save(ChuanyunMemberHourDO.SCHEMA_CODE,selfMonthString,true);
                    }
                    if(chuanyunSaveDTO.getSuccessful()){
//                    log.info("新增工时标准化成功");
                    }else{
                        log.warn("新增工时标准化失败");
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                log.error(""+self);
                e.printStackTrace();
            }
        });
    }

    /**
     * 员工每月、各个项目标准工时会职工
     * @param chuanyunUserCompanyDO 员工信息
     */
    private void standardMonthlyHour(ChuanyunUserCompanyDO chuanyunUserCompanyDO) {
        var objectMapper=new ObjectMapper();
        //每月每项目工时,按月-项目-用户分组统计标准工时
        List<ChuanyunSelfWorkHourDO> list = chuanyunSelfWorkHourDao.getMonthProjectHour(chuanyunUserCompanyDO.getUserId());
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        list.forEach(projectMonth->{
            //获取人员-月度-相同工时状态标准工时汇总
            ChuanyunSelfWorkHourDO monthHour = chuanyunSelfWorkHourDao.getMonthHour(projectMonth.getUserId(),df.format(projectMonth.getDayLogDate()),projectMonth.getStatus());

            //获取人员月度工资信息
            //TODO 张曦铭的薪资信息同步之后需要使用U9中的数据更新薪资信息
            ChuanyunPayrollDO chuanyunPayrollDO = chuanyunPayrollDao.getPayroll(projectMonth.getUserId(),df.format(projectMonth.getDayLogDate()));
            ChuanyunSelfCostDO chuanyunSelfCostDO = new ChuanyunSelfCostDO();
            ChuanyunMemberMonthCostDTO chuanyunMemberMonthCostDTO = new ChuanyunMemberMonthCostDTO();
            if (chuanyunPayrollDO != null) {
                BigDecimal cost = (projectMonth.getStandardWorkHour().divide(monthHour.getStandardWorkHour(), 2, RoundingMode.HALF_UP)).multiply(chuanyunPayrollDO.getWagesPayable());
                chuanyunSelfCostDO.setCost(cost);
                chuanyunMemberMonthCostDTO.setCost(cost);
            }else{
                chuanyunSelfCostDO.setCost(BigDecimal.ZERO);
                chuanyunMemberMonthCostDTO.setCost(BigDecimal.ZERO);
            }
            chuanyunSelfCostDO.setStatus(projectMonth.getStatus());
            chuanyunSelfCostDO.setMonthlyTotalHour(monthHour.getStandardWorkHour());
            chuanyunSelfCostDO.setProjectId(projectMonth.getProjectId());
            chuanyunSelfCostDO.setProjectType(projectMonth.getProjectType());
            chuanyunSelfCostDO.setUserId(projectMonth.getUserId());
//            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-01 HH:mm:ss");
//            String newDate = dateTimeFormatter.format(projectMonth.getDayLogDate());
//            LocalDateTime dt = LocalDateTime.parse(newDate,df);
            chuanyunSelfCostDO.setDayLogDate(projectMonth.getDayLogDate().withDayOfMonth(1));
            chuanyunSelfCostDO.setTotalHour(projectMonth.getStandardWorkHour());
            ChuanyunSelfCostDO oldSelfDO = chuanyunSelfCostDao.getOldSelfDO(chuanyunSelfCostDO.getUserId(),chuanyunSelfCostDO.getProjectId(),df.format(chuanyunSelfCostDO.getDayLogDate()));
            if (oldSelfDO !=null) {
                chuanyunSelfCostDao.delete(oldSelfDO);
            }
            chuanyunSelfCostDao.save(chuanyunSelfCostDO);


            //更新氚云数据
            chuanyunMemberMonthCostDTO.setProjectId(projectMonth.getProjectId());
            chuanyunMemberMonthCostDTO.setProjectType(projectMonth.getProjectType());
            chuanyunMemberMonthCostDTO.setStatus(projectMonth.getStatus());
            chuanyunMemberMonthCostDTO.setUserId(projectMonth.getUserId());
            chuanyunMemberMonthCostDTO.setMonthWorkHour(monthHour.getStandardWorkHour());
            chuanyunMemberMonthCostDTO.setDayLogDate(projectMonth.getDayLogDate().withDayOfMonth(1));
            chuanyunMemberMonthCostDTO.setWorkHour(projectMonth.getStandardWorkHour());
            chuanyunMemberMonthCostDTO.setDepartmentId(chuanyunUserCompanyDO.getDepartmentId());
            try {
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000003_2,"+projectMonth.getProjectId());
                matchers.add("F0000009_2,"+projectMonth.getStatus());
                matchers.add("F0000002_2,"+projectMonth.getUserId());
                matchers.add("F0000004_2,"+projectMonth.getDayLogDate().withDayOfMonth(1));
                var filter= Filter.instance(0,1,true,"And",matchers);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunMemberMonthCostDO.SCHEMA_CODE,filter);
                var memberMonthString=objectMapper.writeValueAsString(chuanyunMemberMonthCostDTO);
                ChuanyunSaveDTO chuanyunSaveDTO;
                if (chuanyunFindAllResponse.getReturnData()!=null){
                    List<ChuanyunMemberMonthCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                    ChuanyunMemberMonthCostDO firstResult = result.get(0);
                    chuanyunSaveDTO =chuanYunManager.update(ChuanyunMemberMonthCostDO.SCHEMA_CODE,firstResult.getObjectId(),memberMonthString);
                }else {
                    chuanyunSaveDTO =chuanYunManager.save(ChuanyunMemberMonthCostDO.SCHEMA_CODE,memberMonthString,true);
                }
                if(chuanyunSaveDTO.getSuccessful()){
//                    log.info("新增项目成员月度成本汇总成功");
                }else{
                    log.warn("新增项目成员月度成本汇总失败");
                }
            } catch (JsonProcessingException | BigSizeException e) {
                e.printStackTrace();
                log.warn("########"+projectMonth+"");
            }
        });
    }
}

@Data
class ChuanyunMemberMonthCostDTO{
    /**
     * 项目ID
     */
    @JsonProperty("F0000003")
    private String projectId;

    /**
     * 项目类型
     */
    @JsonProperty("F0000008")
    private String projectType;

    /**
     * 日志流程状态
     */
    @JsonProperty("F0000009")
    private String status;

    /**
     * 用户ID
     */
    @JsonProperty("F0000002")
    private String userId;

    /**
     * 日期
     */
    @JsonProperty("F0000004")
    @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime dayLogDate;

    /**
     * 工时
     */
    @JsonProperty("F0000005")
    private BigDecimal workHour;

    /**
     * 成本
     */
    @JsonProperty("F0000006")
    private BigDecimal cost;

    /**
     * 月度工时汇总
     */
    @JsonProperty("F0000007")
    private BigDecimal monthWorkHour;

    /**
     * 部门Id
     */
    @JsonProperty("F0000010")
    private String departmentId;
}

@Data
class ChuanyunProjectMemberCostDTO {

    /**
     * 项目ID
     */
    @JsonProperty("F0000001")
    private String projectId;

    /**
     * 项目类型
     */
    @JsonProperty("F0000005")
    private String projectType;

    /**
     * 用户ID
     */
    @JsonProperty("F0000002")
    private String userId;

    /**
     * 工时
     */
    @JsonProperty("F0000003")
    private BigDecimal workHour;

    /**
     * 成本
     */
    @JsonProperty("F0000004")
    private BigDecimal cost;

    /**
     * 总人天
     */
    @JsonProperty("F0000006")
    private BigDecimal personDays;

    /**
     * 工时状态
     */
    @JsonProperty("F0000007")
    private String status;
}

/**
 * 项目汇总工时
 */
@Data
class ChuanyunProjectTotalCostDTO{

    /**
     * 项目ID
     */
    @JsonProperty("F0000001")
    private String projectId;

    /**
     * 项目类型
     */
    @JsonProperty("F0000004")
    private String projectType;

    /**
     * 工时
     */
    @JsonProperty("F0000003")
    private BigDecimal workHour;

    /**
     * 成本
     */
    @JsonProperty("F0000002")
    private BigDecimal cost;

    /**
     * 总人天
     */
    @JsonProperty("F0000005")
    private BigDecimal personDays;

    /**
     * 工时状态
     */
    @JsonProperty("F0000006")
    private String status;

    @JsonProperty("F0000007")
    private BigDecimal amount;
}

@Data
class ChuanyunAdjustAccountDTO {
    @JsonProperty("F0000002")
    private String projectCode;

    /**
     * 核算实施人天
     */
    @JsonProperty("F0000003")
    private BigDecimal personDays;

    /**
     * 核算实施人力成本
     */
    @JsonProperty("F0000004")
    private BigDecimal laborCost;

    /**
     * 核算实施费用
     */
    @JsonProperty("F0000005")
    private BigDecimal implementationCosts;

    /**
     *核算实施总成本
     */
    @JsonProperty("F0000006")
    private BigDecimal totalImplementationCosts;

    /**
     * 项目
     */
    @JsonProperty("F0000007")
    private String projectId;
}

/**
 * 工时标准化更新数据
 */
@Data
class ChuanyunSelfWorkHourDTO {

    @JsonProperty("F0000002")
    private String userId;

    @JsonProperty("F0000001")
    private String projectId;

    /**
     * 项目类型
     */
    @JsonProperty("F0000005")
    private String projectType;

    /**
     * 流程状态
     */
    @JsonProperty("F0000006")
    private String status;

    @JsonProperty("F0000003")
    @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime dayLogDate;

    @JsonProperty("F0000004")
    private BigDecimal standardWorkHour;

    /**
     * 部门Id
     */
    @JsonProperty("F0000007")
    private String departmentId;
}

